@@ -6,7 +6,7 @@
 //! Utility to run a regtest bitcoind process, useful in integration testing environment
 //!
 //! ```no_run
-//! use core_rpc::RpcApi;
+//! use bitcoincore_rpc::RpcApi;
 //! let bitcoind = bitcoind::BitcoinD::new("/usr/local/bin/bitcoind").unwrap();
 //! assert_eq!(0, bitcoind.client.get_blockchain_info().unwrap().blocks);
 //! ```
@@ -20,11 +20,11 @@ use std::ffi::OsStr;
 use std::net::{Ipv4Addr, SocketAddrV4, TcpListener};
 use std::path::PathBuf;
 use std::process::{Child, Command, ExitStatus, Stdio};
-use std::thread;
 use std::time::Duration;
+use std::{env, thread};
 use tempfile::TempDir;
 
-pub extern crate core_rpc as bitcoincore_rpc;
+pub extern crate bitcoincore_rpc;
 pub extern crate tempfile;
 
 /// Struct representing the bitcoind process with related information
@@ -80,7 +80,9 @@ const LOCAL_IP: Ipv4Addr = Ipv4Addr::new(127, 0, 0, 1);
 
 /// The node configuration parameters, implements a convenient [Default] for most common use.
 ///
-/// `#[non_exhaustive]` allows adding new parameters without breaking downstream users
+/// `#[non_exhaustive]` allows adding new parameters without breaking downstream users.
+/// Users cannot instantiate the struct directly, they need to create it via the `default()` method
+/// and mutate fields according to their preference.
 ///
 /// Default values:
 /// ```
@@ -89,6 +91,7 @@ const LOCAL_IP: Ipv4Addr = Ipv4Addr::new(127, 0, 0, 1);
 /// conf.view_stdout = false;
 /// conf.p2p = bitcoind::P2P::No;
 /// conf.network = "regtest";
+/// conf.tmpdir = None;
 /// assert_eq!(conf, bitcoind::Conf::default());
 /// ```
 ///
@@ -109,6 +112,13 @@ pub struct Conf<'a> {
     /// Must match what specified in args without dashes, needed to locate the cookie file
     /// directory with different/esoteric networks
     pub network: &'a str,
+
+    /// Optionally specify the root of where the temporary directories will be created.
+    /// If none and the env var `TEMPDIR_ROOT` is set, the env var is used.
+    /// If none and the env var `TEMPDIR_ROOT` is not set, the default temp dir of the OS is used.
+    /// It may be useful for example to set to a ramdisk so that bitcoin nodes spawn very fast
+    /// because their datadirs are in RAM
+    pub tmpdir: Option<PathBuf>,
 }
 
 impl Default for Conf<'_> {
@@ -118,6 +128,7 @@ impl Default for Conf<'_> {
             view_stdout: false,
             p2p: P2P::No,
             network: "regtest",
+            tmpdir: None,
         }
     }
 }
@@ -132,8 +143,15 @@ impl BitcoinD {
 
     /// Launch the bitcoind process from the given `exe` executable with given [Conf] param
     pub fn with_conf<S: AsRef<OsStr>>(exe: S, conf: &Conf) -> Result<BitcoinD, Error> {
-        let _work_dir = TempDir::new()?;
-        let datadir = _work_dir.path().to_path_buf();
+        let work_dir = match &conf.tmpdir {
+            Some(path) => TempDir::new_in(path),
+            None => match env::var("TEMPDIR_ROOT") {
+                Ok(env_path) => TempDir::new_in(env_path),
+                Err(_) => TempDir::new(),
+            },
+        }?;
+        debug!("work_dir: {:?}", work_dir);
+        let datadir = work_dir.path().to_path_buf();
         let cookie_file = datadir.join(conf.network).join(".cookie");
         let rpc_port = get_available_port()?;
         let rpc_socket = SocketAddrV4::new(LOCAL_IP, rpc_port);
@@ -205,7 +223,7 @@ impl BitcoinD {
         Ok(BitcoinD {
             process,
             client,
-            _work_dir,
+            _work_dir: work_dir,
             params: ConnectParams {
                 datadir,
                 cookie_file,
@@ -298,6 +316,8 @@ pub fn downloaded_exe_path() -> Option<String> {
 
 #[cfg(test)]
 mod test {
+    use crate::bitcoincore_rpc::jsonrpc::serde_json::Value;
+    use crate::bitcoincore_rpc::Client;
     use crate::downloaded_exe_path;
     use crate::{get_available_port, BitcoinD, Conf, LOCAL_IP, P2P};
     use bitcoincore_rpc::RpcApi;
@@ -312,15 +332,6 @@ mod test {
         assert_eq!(format!("127.0.0.1:{}", port), format!("{}", socket));
     }
 
-    #[test]
-    fn test_default() {
-        let conf = Conf {
-            p2p: P2P::Yes,
-            ..Default::default()
-        };
-        assert_eq!("regtest", conf.network);
-    }
-
     #[test]
     fn test_bitcoind() {
         let exe = init();
@@ -354,52 +365,44 @@ mod test {
     #[test]
     fn test_p2p() {
         let exe = init();
-        let conf = Conf {
-            p2p: P2P::Yes,
-            ..Default::default()
-        };
+        let mut conf = Conf::default();
+        conf.p2p = P2P::Yes;
+
         let bitcoind = BitcoinD::with_conf(&exe, &conf).unwrap();
-        assert_eq!(bitcoind.client.get_peer_info().unwrap().len(), 0);
-        let other_conf = Conf {
-            p2p: bitcoind.p2p_connect(false).unwrap(),
-            ..Default::default()
-        };
+        assert_eq!(peers_connected(&bitcoind.client), 0);
+        let mut other_conf = Conf::default();
+        other_conf.p2p = bitcoind.p2p_connect(false).unwrap();
+
         let other_bitcoind = BitcoinD::with_conf(&exe, &other_conf).unwrap();
-        assert_eq!(bitcoind.client.get_peer_info().unwrap().len(), 1);
-        assert_eq!(other_bitcoind.client.get_peer_info().unwrap().len(), 1);
+        assert_eq!(peers_connected(&bitcoind.client), 1);
+        assert_eq!(peers_connected(&other_bitcoind.client), 1);
     }
 
     #[test]
     fn test_multi_p2p() {
-        let conf_node1 = Conf {
-            p2p: P2P::Yes,
-            ..Default::default()
-        };
+        let mut conf_node1 = Conf::default();
+        conf_node1.p2p = P2P::Yes;
         let node1 = BitcoinD::with_conf(exe_path(), &conf_node1).unwrap();
 
         // Create Node 2 connected Node 1
-        let conf_node2 = Conf {
-            p2p: node1.p2p_connect(true).unwrap(),
-            ..Default::default()
-        };
+        let mut conf_node2 = Conf::default();
+        conf_node2.p2p = node1.p2p_connect(true).unwrap();
         let node2 = BitcoinD::with_conf(exe_path(), &conf_node2).unwrap();
 
-        // Create Node 3 Connected To Node 2
-        let conf_node3 = Conf {
-            p2p: node2.p2p_connect(false).unwrap(),
-            ..Default::default()
-        };
+        // Create Node 3 Connected To Node
+        let mut conf_node3 = Conf::default();
+        conf_node3.p2p = node2.p2p_connect(false).unwrap();
         let node3 = BitcoinD::with_conf(exe_path(), &conf_node3).unwrap();
 
         // Get each nodes Peers
-        let node1_peers = node1.client.get_peer_info().unwrap();
-        let node2_peers = node2.client.get_peer_info().unwrap();
-        let node3_peers = node3.client.get_peer_info().unwrap();
+        let node1_peers = peers_connected(&node1.client);
+        let node2_peers = peers_connected(&node2.client);
+        let node3_peers = peers_connected(&node3.client);
 
         // Peers found
-        assert!(node1_peers.len() >= 1);
-        assert!(node2_peers.len() >= 1);
-        assert_eq!(node3_peers.len(), 1, "listen false but more than 1 peer");
+        assert!(node1_peers >= 1);
+        assert!(node2_peers >= 1);
+        assert_eq!(node3_peers, 1, "listen false but more than 1 peer");
     }
 
     #[cfg(not(any(feature = "0_17_1", feature = "0_18_0", feature = "0_18_1")))]
@@ -458,6 +461,11 @@ mod test {
         );
     }
 
+    fn peers_connected(client: &Client) -> usize {
+        let result: Vec<Value> = client.call("getpeerinfo", &[]).unwrap();
+        result.len()
+    }
+
     fn exe_path() -> String {
         if let Some(downloaded_exe_path) = downloaded_exe_path() {
             downloaded_exe_path
